/* Marek Handl
 * CS321 - Computer Graphics
 * Lab 5
 * October 2006
 */

#include <iostream>
#include <vector>
#include <sstream>
#include <fstream>
#include <qevent.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qwidget.h>
#include <qapplication.h>
#include <qhbuttongroup.h>
#include <qhgroupbox.h>
#include <qcolordialog.h>
#include <qstringlist.h>
#include <qcombobox.h>
#include <qvgroupbox.h>
#include <ctype.h>
#include "Lab3DerivedShell.h"
#include "image.h"
#include "shape.h"
#include "point.h"
#include "line.h"
#include "variable_shape.h"
#include "polygon.h"
#include "bezier.h"
#include "matrix.h"
/*#include "rectangle.h"
#include "circle.h"*/

using namespace std;

// constructor
Lab3DerivedShell::Lab3DerivedShell( QWidget* parent, const std::string& name ) : Shell( parent, name )
{
	areaWidth = 500;
	areaHeight = 350;
	// calls Shell constructor at first
	setGeometry( 610, 20, areaWidth+125, areaHeight+240 );	// resize the window a little bit
	// Create the lower button area
	QHButtonGroup* lowerButtons = new QHButtonGroup(this);
	assert(lowerButtons);
	// Add buttons to the group and connect them to slots
	// Point button
	pointButton = new QPushButton("Point", lowerButtons);
	assert(pointButton);
	QObject::connect(pointButton, SIGNAL(clicked()), this, SLOT(PointSlot()));
	pointButton->setMinimumSize(pointButton->sizeHint());
	// Line button
	lineButton = new QPushButton("Line", lowerButtons);
	assert(lineButton);
	QObject::connect(lineButton, SIGNAL(clicked()), this, SLOT(LineSlot()));
	lineButton->setMinimumSize(lineButton->sizeHint());
	// Polygon button
	polygonButton = new QPushButton("Polygon", lowerButtons);
	assert(pointButton);
	QObject::connect(polygonButton, SIGNAL(clicked()), this, SLOT(PolygonSlot()));
	polygonButton->setMinimumSize(polygonButton->sizeHint());
	// Bezier Curve button
	bezierButton = new QPushButton("Bezier", lowerButtons);
	assert( bezierButton);
	QObject::connect( bezierButton, SIGNAL(clicked()), this, SLOT( BezierSlot() ) );
	bezierButton->setMinimumSize( bezierButton->sizeHint() );

	// add buttons to a layout
	QHBoxLayout* buttonsLayout = new QHBoxLayout();
	assert(buttonsLayout);
	buttonsLayout->addWidget( lowerButtons );
	// add buttons to main layout
	mainLayout->addLayout( buttonsLayout );

	// create bottom line of widgets
	QHGroupBox* bottomWidgets = new QHGroupBox(this);
	assert(bottomWidgets);
	// non-zero winding algorithm check-box
	nzwCheckBox = new QCheckBox("Non-zero Winding", bottomWidgets );
	assert(nzwCheckBox);
	QObject::connect(nzwCheckBox, SIGNAL(clicked()), this, SLOT(NzwSlot()));
	// set line color button
	QPushButton *colorButton = new QPushButton("Color", bottomWidgets);
	assert(colorButton);
	QObject::connect(colorButton, SIGNAL(clicked()), this, SLOT(ColorSlot()));
	colorButton->setMinimumSize(colorButton->sizeHint());
	// indication of selected color
	colorIndication = new QHBox( bottomWidgets );
	assert( colorIndication );
	colorIndication->setFixedHeight( 20 );
	colorIndication->setFixedWidth( 10 );	
	// set fill color button
	QPushButton *fillColorButton = new QPushButton("Fill Color", bottomWidgets);
	assert(fillColorButton);
	QObject::connect(fillColorButton, SIGNAL(clicked()), this, SLOT(FillColorSlot()));
	fillColorButton->setMinimumSize(fillColorButton->sizeHint());
	// indication of selected fill color
	fillColorIndication = new QHBox( bottomWidgets );
	assert( fillColorIndication );
	fillColorIndication->setFixedHeight( 20 );
	fillColorIndication->setFixedWidth( 10 );	
	// Line Style ListBox
	QStringList lineStyles;
	lineStyles = "Solid";
	lineStyles << "Dash" << "Dot" << "DashDot" << "DashDotDot";
	lineStyleBox = new QComboBox(bottomWidgets);
	assert(lineStyleBox);
	lineStyleBox->insertStringList( lineStyles, 0 );	
	QObject::connect(lineStyleBox, SIGNAL(activated(int)), this, SLOT(LineStyleSlot()));
	// Fill Style ListBox
	QStringList fillStyles;
	fillStyles = "NoBrush";
	fillStyles << "Solid" << "Dense1" << "Dense2" << "Dense3" << "Dense4" << "Dense5" << "Dense6" << "Dense7" << "Hor" << "Ver" << "Cross" << "BDiag" << "FDiag" << "DiagCross";
	fillStyleBox = new QComboBox(bottomWidgets);
	assert(fillStyleBox);
	fillStyleBox->insertStringList( fillStyles, 0 );
	QObject::connect(fillStyleBox, SIGNAL(activated(int)), this, SLOT(FillStyleSlot()));
	// add bottom line to a layout
	QHBoxLayout* bottomLayout = new QHBoxLayout();
	assert(bottomLayout);
	bottomLayout->addWidget( bottomWidgets );
	// add bottom widgets to main layout
	mainLayout->addLayout( bottomLayout );

	// create line of transformation widgets
	QHGroupBox* transformationWidgets = new QHGroupBox(this);
	assert( transformationWidgets );
	QVGroupBox* panWidgets = new QVGroupBox(this);
	assert( panWidgets );
	// slider for translation in X axis
	panX = new QSlider( -3*areaWidth, 3*areaWidth, areaWidth/10, 0, Qt::Horizontal, panWidgets );
	assert( panX );
	QObject::connect( panX, SIGNAL( valueChanged(int) ), this, SLOT( PanXSlot() ) );
	panX->setTickmarks( QSlider::Above );
	panX->setTickInterval( areaWidth/2 );
	// slider for translation in Y axis
	panY = new QSlider( -3*areaHeight, 3*areaHeight, areaHeight/10, 0, Qt::Horizontal, panWidgets );
	assert( panY );
	QObject::connect( panY, SIGNAL( valueChanged(int) ), this, SLOT( PanYSlot() ) );
	panY->setTickmarks( QSlider::Above );
	panY->setTickInterval( areaHeight/2 );
	
	QVBoxLayout *panLayout = new QVBoxLayout();
	assert( panLayout );
	panLayout->addWidget( panWidgets );
	// slider for rotation
	rotation = new QSlider( -180, 180, 5, 0, Qt::Horizontal, transformationWidgets );
	assert( rotation );
	QObject::connect( rotation, SIGNAL( valueChanged(int) ), this, SLOT( RotationSlot() ) );
	rotation->setTickmarks( QSlider::Above );
	rotation->setTickInterval( 45 );

	QHBoxLayout* transformationLayout = new QHBoxLayout();
	assert( transformationLayout );
	transformationLayout->addLayout( panLayout );
	transformationLayout->addWidget( transformationWidgets );
	mainLayout->addLayout( transformationLayout );
	
	// ****** end of window design *****
	act_image = new image( "default_image" );
	images.push_back( act_image );	// adds new image to vector of images
	mode = 'p'; 	// implicit is point drawing mode
	pointButton->setDown( TRUE );	// push down the Point button
	buttonDown = false;	// mouse button has not been pushed yet (used when drawing lines)
	drawingBezier = drawingPolygon = false;
	nonZeroWinding = TRUE;
	nzwCheckBox->setChecked( nonZeroWinding );
	pol_ptr = NULL;
	bez_ptr = NULL;
	penStyle = Qt::SolidLine;
	lineStyleBox->setCurrentItem( (int) penStyle - 1);
	brushStyle = Qt::SolidPattern;
	fillStyleBox->setCurrentItem( (int) brushStyle );
	brushColor = color = QColor( 255, 255, 255 );
	colorIndication->setBackgroundColor( color );
	fillColorIndication->setBackgroundColor( brushColor );
	viewContext = new ViewContext();
}

// destructor
Lab3DerivedShell::~Lab3DerivedShell() {
	if ( pol_ptr != NULL ) delete pol_ptr;
	for (int i=0; i<images.size(); i++)
		delete images[i];	// free memory allocated by each image (calls image destructors which delete all shapes at first)
	images.clear();		// clear the vector of images
}

// erase image, in memory and on screen
void Lab3DerivedShell::Erase()
{
  	StopDrawingPolygonOrBezier();
	buttonDown = false;
	act_image->Erase();	// delete all shapes in image (deletes them from data structure not from screen)
	drawArea->erase();	// erase draw area (deletes them from screen)
	viewContext->Reset();	// resets all panning, rotation and zooming
	panX->setValue( 0 );
	panY->setValue( 0 );
	rotation->setValue( 0 );
}

// Command line parsing and executing wanted commands
bool Lab3DerivedShell::CmdParse(const std::string& cmd)
{
	string type, x_string, y_string, z_string, zz_string;
	int x, y, z, x1, y1;
	StopDrawingPolygonOrBezier();
	stringstream ss( cmd, stringstream::out | stringstream::in );	// for string parsing
	ss >> type >> x_string >> y_string >> z_string >> zz_string;
	// convert the command line input to lower case
	std::transform( type.begin(), type.end(), type.begin(), (int(*)(int)) tolower);		// casting is neccesary to choose the right tolower
	std::transform( x_string.begin(), x_string.end(), x_string.begin(), (int(*)(int)) tolower);
	std::transform( y_string.begin(), y_string.end(), y_string.begin(), (int(*)(int)) tolower);
	std::transform( z_string.begin(), z_string.end(), z_string.begin(), (int(*)(int)) tolower);
	// Erase Image
	if ( type=="e" ) { 
		Erase();
		return true;
	}
	// Quit
	else if ( type=="q" ) { 
		exit(0);
	}
	// Write image to a file
	else if ( type=="s" && x_string!="" ) { 
		WriteFile( x_string );
		return true;		
	}
	// Load image from file
	else if ( type=="l" && x_string!="" ) {
		viewContext->Reset();
		ReadFile( x_string );
		return true;
	}
	// New Point
	else if ( (type=="point" ) && x_string!= "" && y_string!="" ) {	// three values on the line, separated by spaces
		if ( (x = atoi(x_string.c_str()))!=0 && (y = atoi(y_string.c_str()))!=0 ) {
			cout << "point <" << x << ", " << y << ">" << endl;
			shape* sh_ptr = new point( x, y, penStyle, color  );	// create new point
			act_image->Add( sh_ptr );	// add point to image data structure
			sh_ptr->Draw( drawArea, viewContext );	// draw the point
			return true;
		}
	}
	// New Line
	else if ( (type=="line") && x_string!= "" && y_string!="" && z_string!="" && zz_string!="" ) {	// three values on the line, separated by spaces
		if ( (x = atoi(x_string.c_str()))!=0 && (y = atoi(y_string.c_str()))!=0 && (x1=atoi(z_string.c_str()))!=0 && (y1=atoi(zz_string.c_str()))!=0 ) {
			cout << "line <" << x << ", " << y << "> <" << x1 << ", " << y1 << ">"<< endl;
			shape* sh_ptr = new line( x, y, x1, y1, penStyle, color  );	// create new point
			act_image->Add( sh_ptr );	// add point to image data structure
			sh_ptr->Draw( drawArea, viewContext );	// draw the point
			return true;
		}
	}
	// Show Help window
	else if ( type=="h" ) {
		HelpSlot();
		return true;
	}
	// Set lines color
	else if ( type=="setc" && x_string!="" && y_string!="" && z_string!="" ) {
		if ( ( (x=atoi(x_string.c_str()))!=0 || x_string=="0" ) && ( (y=atoi(y_string.c_str()))!=0 || y_string=="0" ) && ( (z=atoi(z_string.c_str()))!=0 || z_string=="0" ) ) {	// conversion from strings to 3 integers (zero is a special case)
			color = QColor( x, y, z );
			cout << "Color set to RGB( " << x << ", " << y << ", " << z << " )" << endl;
			return true;
		}
	}
	// Set filling color
	else if ( type=="setfc" && x_string!="" && y_string!="" && z_string!="" ) {
		if ( ( (x=atoi(x_string.c_str()))!=0 || x_string=="0" ) && ( (y=atoi(y_string.c_str()))!=0 || y_string=="0" ) && ( (z=atoi(z_string.c_str()))!=0 || z_string=="0" ) ) {	// conversion from strings to 3 integers (zero is a special case)
			brushColor = QColor( x, y, z );
			cout << "Filling color set to RGB( " << x << ", " << y << ", " << z << " )" << endl;
			return true;
		}
	}
	// Set pen style (line style)
	else if ( type=="setp" && x_string!="" ) {
		if ( x_string=="solid" ) {
			penStyle = Qt::SolidLine;
			lineStyleBox->setCurrentItem( (int) penStyle - 1);
			cout << "Pen style set to SolidLine" << endl;
			return true;
		}
		if ( x_string=="dash" ) {
			penStyle = Qt::DashLine;
			lineStyleBox->setCurrentItem( (int) penStyle - 1);
			cout << "Pen style set to DashLine" << endl;
			return true;
		}
		if ( x_string=="dot" ) {
			penStyle = Qt::DotLine;
			lineStyleBox->setCurrentItem( (int) penStyle - 1);
			cout << "Pen style set to DotLine" << endl;
			return true;
		}
		if ( x_string=="dashdot" ) {
			penStyle = Qt::DashDotLine;
			lineStyleBox->setCurrentItem( (int) penStyle - 1);
			cout << "Pen style set to DashDotLine" << endl;
			return true;
		}
		if ( x_string=="dashdotdot" ) {
			penStyle = Qt::DashDotDotLine;
			lineStyleBox->setCurrentItem( (int) penStyle - 1);
			cout << "Pen style set to DashDotDotLine" << endl;
			return true;
		}
	}
	// Set filling style
	else if ( type=="setf" && x_string!="" ) {
		if ( x_string=="solid" ) {
			brushStyle = Qt::SolidPattern;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to SolidPattern" << endl;
			return true;
		}
		if ( x_string=="dense1" ) {
			brushStyle = Qt::Dense1Pattern;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to Dense1Pattern" << endl;
			return true;
		}
		if ( x_string=="dense2" ) {
			brushStyle = Qt::Dense2Pattern;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to Dense2Pattern" << endl;
			return true;
		}
		if ( x_string=="dense3" ) {
			brushStyle = Qt::Dense3Pattern;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to Dense3Pattern" << endl;
			return true;
		}
		if ( x_string=="dense4" ) {
			brushStyle = Qt::Dense4Pattern;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to Dense4Pattern" << endl;
			return true;
		}
		if ( x_string=="dense5" ) {
			brushStyle = Qt::Dense5Pattern;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to Dense5Pattern" << endl;
			return true;
		}
		if ( x_string=="dense6" ) {
			brushStyle = Qt::Dense6Pattern;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to Dense6Pattern" << endl;
			return true;
		}
		if ( x_string=="dense7" ) {
			brushStyle = Qt::Dense7Pattern;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to Dense7Pattern" << endl;
			return true;
		}
		if ( x_string=="hor" ) {
			brushStyle = Qt::HorPattern;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to HorPattern" << endl;
			return true;
		}
		if ( x_string=="ver" ) {
			brushStyle = Qt::VerPattern;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to VerPattern" << endl;
			return true;
		}
		if ( x_string=="cross" ) {
			brushStyle = Qt::CrossPattern;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to CrossPattern" << endl;
			return true;
		}
		if ( x_string=="bdiag" ) {
			brushStyle = Qt::BDiagPattern;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to BDiagPattern" << endl;
			return true;
		}
		if ( x_string=="fdiag" ) {
			brushStyle = Qt::FDiagPattern;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to FDiagPattern" << endl;
			return true;
		}
		if ( x_string=="diagcross" ) {
			brushStyle = Qt::DiagCrossPattern;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to DiagCrossPattern" << endl;
			return true;
		}
		if ( x_string=="nobrush" ) {
			brushStyle = Qt::NoBrush;
			fillStyleBox->setCurrentItem( (int) brushStyle );
			cout << "Filling style set to NoBrush" << endl;
			return true;
		}
	}
	// Change drawing mode (point, line, polygon...)
	else if ( type=="setm" && x_string!="" ) {
		if ( x_string=="point" ) {
			PointSlot();
			return true;
		}
		if ( x_string=="line" ) {
			LineSlot();
			return true;
		}
		if ( x_string=="polygon" ) {
			PolygonSlot();
			return true;
		}
		if ( x_string=="bezier" ) {
			BezierSlot();
			return true;
		}
	}
	// Switch on/off using non-zero winding rule for polygon filling
	else if ( type=="nzw" && x_string!="" ) {
		if ( x_string=="on" ) {
			nonZeroWinding = TRUE;
			cout << "Non-zero Winding algorithm is ON." << endl;
			nzwCheckBox->setChecked( nonZeroWinding );
			return true;
		}
		if ( x_string=="off" ) {
			nonZeroWinding = FALSE;
			cout << "Non-zero Winding algorithm is OFF." << endl;
			nzwCheckBox->setChecked( nonZeroWinding );
			return true;
		}
	}
	// refreshing screen - used for debugging
	else if (type=="r") {
		Refresh();
		return true;
	}
	// cancel all translations, rotation and scaling - used for debugging
	else if (type=="rs") {
		viewContext->Reset();
		// move sliders to default values without calling their slots
		panX->blockSignals( true );
		panY->blockSignals( true );
		rotation->blockSignals( true );
		panX->setValue( 0 );
		panY->setValue( 0 );
		rotation->setValue( 0 );
		panX->blockSignals( false );
		panY->blockSignals( false );
		rotation->blockSignals( false );
		Refresh();
		return true;
	}
	cout << "Wrong input" << endl;
	return false;
}

// when an event occurs (e.g. Paint, Mouse events...) this method is called
void Lab3DerivedShell::Draw(QEvent* event)
{
	// Paint event
	if ( event->type() == QEvent::Paint ) {
		buttonDown = false;
		StopDrawingPolygonOrBezier();
		act_image->ReDraw( drawArea, viewContext );
	}
	// Mouse events follow:

	// point drawing mode
	else if ( mode == 'p' ) {
		// on mouse button down event a point is drawn in the mouse position
		if ( event->type() == QEvent::MouseButtonPress ) {
			QPoint p = viewContext->WindowToModel( QPoint( (((QMouseEvent*) event)->x()), ((QMouseEvent*) event)->y()) );
			int x = p.x();
			int y = p.y();
			cout << "point <" << x << ", " << y << ">" << endl;
			shape* sh_ptr = new point( x, y, penStyle, color );	// create a new point object
			act_image->Add( sh_ptr );		// add point in vector of shapes in actual image
			sh_ptr->Draw( drawArea, viewContext );		// draw point on screen
		}
	}
	// line drawing mode
	else if ( mode == 'l' ) {
		// start point - mouse button pressed
		if ( event->type() == QEvent::MouseButtonPress && buttonDown == false ) {
			buttonDown = true;
			QPoint p = viewContext->WindowToModel( QPoint( (((QMouseEvent*) event)->x()), ((QMouseEvent*) event)->y()) );
			start_x = p.x();
			start_y = p.y();
			p = viewContext->ModelToWindow( p );	// conversion back and forth necessary because of round-off error
			DrawTempPixel( p.x(), p.y() );
		}
		// end point - mouse button released
		else if ( event->type() == QEvent::MouseButtonRelease && buttonDown == true ) {
			buttonDown = false;
			QPoint p = viewContext->WindowToModel( QPoint( (((QMouseEvent*) event)->x()), ((QMouseEvent*) event)->y()) );
			int x = p.x();
			int y = p.y();
			cout << "line <" << start_x << ", " << start_y << "> <" << x << ", " << y << ">" << endl;
			shape* sh_ptr = new line( start_x, start_y, x, y, penStyle, color );
			act_image->Add( sh_ptr );
			sh_ptr->Draw( drawArea, viewContext );
		}
	}
	// polygon drawing mode
	else if ( mode == 'g' ) {
		// polygon finished - double click
		if ( event->type() == QEvent::MouseButtonDblClick && drawingPolygon ) {
			act_image->Add( pol_ptr );
			pol_ptr->Draw( drawArea, viewContext );
			pol_ptr = NULL;
			StopDrawingPolygonOrBezier();
			cout << "polygon" << endl;
		}
		// add a new vertex to a polygon
		else if ( event->type() == QEvent::MouseButtonPress ) {
			if ( !drawingPolygon) {	// start drawing a new polygon
				drawingPolygon = true;
				polygonButton->setText( "*Polygon*" );	// show to user that he started drawing a polygon
				cout << "drawing polygon started" << endl;
				if ( pol_ptr != NULL ) delete pol_ptr;	// just in case, polygon wasn't removed properly last time
				pol_ptr = new polygon( nonZeroWinding, penStyle, color, brushStyle, brushColor );
			}
			QPoint p = viewContext->WindowToModel( QPoint( (((QMouseEvent*) event)->x()), ((QMouseEvent*) event)->y()) );
			pol_ptr->AddNode( p.x(), p.y() );
			p = viewContext->ModelToWindow( p );	// conversion back and forth necessary because of round-off error
			DrawTempPixel( p.x(), p.y() );	// draw a pixel on screen representing added vertex
		}
	}
	// bezier curve drawing mode
	else if ( mode == 'b' ) {
		// bezier curve finished - double click
		if ( event->type() == QEvent::MouseButtonDblClick && drawingBezier ) {
			act_image->Add( bez_ptr );
			bez_ptr->Draw( drawArea, viewContext );
			bez_ptr = NULL;
			StopDrawingPolygonOrBezier();
			cout << "bezier curve" << endl;
		}
		// add a new vertex to a bezier curve
		else if ( event->type() == QEvent::MouseButtonPress ) {
			if ( !drawingBezier ) {	// start drawing a new bezier
				drawingBezier = true;
				bezierButton->setText( "*Bezier*" );	// show to user that he started drawing a bezier curve
				cout << "drawing bezier curve started" << endl;
				if ( bez_ptr != NULL ) delete bez_ptr;
				bez_ptr = new bezier( 100, penStyle, color );
				QPoint p = viewContext->WindowToModel( QPoint( (((QMouseEvent*) event)->x()), ((QMouseEvent*) event)->y()) );
				bez_ptr->AddNode( p.x(), p.y() );
				p = viewContext->ModelToWindow( p );	// conversion back and forth necessary because of round-off error
				bezierLastPoint = p;
				DrawTempPixel( p.x(), p.y() );	// draw a pixel on screen representing added vertex
			}
			else {
				QPoint p = viewContext->WindowToModel( QPoint( (((QMouseEvent*) event)->x()), ((QMouseEvent*) event)->y()) );
				bez_ptr->AddNode( p.x(), p.y() );
				p = viewContext->ModelToWindow( p );	// conversion back and forth necessary because of round-off error
				// drawing polylines that determine the bezier
				painter.begin(drawArea);
				QPen pen;
				pen.setColor( color );
				pen.setStyle( Qt::SolidLine );
				painter.setPen( pen );
				painter.drawLine( bezierLastPoint, p );
				painter.end();
				bezierLastPoint = p;
			}
		}
	}
	// error
	else {
		cout << "ERROR - invalid mode" << endl;
	}
}

// read image from a file
void Lab3DerivedShell::ReadFile(const std::string& file)
{
	string im_name;		// name of image that is read from file
	ifstream fin;
	fin.open( file.c_str(), fstream::in );	// opens file for reading
	if ( (fin.rdstate() & fstream::failbit) != 0 ) {	// there was an error in opening the file
		cout << endl << "Error opening file! Image has not been loaded." << endl;
		return;
	}
	else {	// file opened correctly, read from it
		Erase();		// delete all shapes from actual image and erases the screen
		getline( fin, im_name );
		act_image->name =  im_name;
		act_image->Read( fin );		// reads shapes parameters from file, creates new shape instances and adds them to image
		fin.close();
		act_image->ReDraw( drawArea, viewContext );
		cout << "Image " << im_name << " loaded from file " << file << "." <<endl;
	}
}

// write image to a file
void Lab3DerivedShell::WriteFile(const std::string& file) const
{
	ofstream fout;
	fout.open( file.c_str(), fstream::out );	// opens file for writing
	if ( (fout.rdstate() & fstream::failbit) != 0 ) {	// there was an error in opening the file
		cout << endl << "Error opening file! image has not been saved." << endl;
		return;
	}
	else {	// file opened correctly, write to it
		fout << act_image->name << endl;	// write name of image
		act_image->Write( fout );	// will call each shapes's write method as well
		fout.close();
		cout << "Image " << act_image->name << " saved to file " << file << "." << endl;
	}
}

// set the string that will be shown in help window
std::string Lab3DerivedShell::HelpText() const
{
	string s;
	s  = "CMD line commands: \n";
	s += " h ..... show this help\n";
	s += " l FILENAME ..... load image from file with name FILENAME\n";
	s += " s FILENAME ...... write image to a file with name FILENAME\n";
	s += " e ..... erase image\n";
	s += " point X Y ..... draw a point with coordinates X, Y\n";
	s += " line X1 Y1 X2 Y2 ... draw a line from <X1,Y1> to <X2,Y2>\n";	
	s += " setc RED GREEN BLUE ... set drawing color (three values from 0 to 255)\n";
	s += " setp STYLE ...... set pen style (values: solid, dash, dot, dashdot, dashdotdot)\n";
	s += " setfc RED GREEN BLUE ... set filling color three (values from 0 to 255)\n";
	s += " setf STYLE ...... set filling style (values: solid, dense1 .. dense7, hor, ver, cross, bdiag, fdiag, diagcross, nobrush)\n";
	s += " setm MODE ....... change drawing mode (values: point, line, polygon, bezier)\n";
	s += " nzw MODE ........ switch on/off non-zero winding algorithm for polygon fill (values: on, off)\n";
	s += " q ..... quit program\n";
	s += "\nMouse Usage: \n";
	s += "Drawing a point - point is drawn in location where you left-click with the mouse.\n";
	s += "Drawing a line - press down left button to set start point, drag the mouse and release to set end point of a line.\n";
	s += "Drawing a polygon - each left-click adds a vertex, polygon is finished by a double-click (stars on polygon button indicate that at least one vertex has been added).\n";
	s += "Drawing a bezier curve - each left-click adds a vertex, bezier curve is finished by a double-click (stars on bezier button indicate that at least one vertex has been added).\n";
	s += "Mouse can be used to select drawing mode - Point, Line, Polygon, Bezier buttons.\n";
	s += "Mouse can be used to select color for drawing lines and filling color, after pushing a button a Color Dialog appers.\n";
	s += "Mouse can be used to select line style and filling pattern using list-boxes.\n";
	s += "\n Commands are not case sensitive.";
	return s;
}

// Point slot
// when Point button is pressed - switch to Point drawing mode
void Lab3DerivedShell::PointSlot()
{
	mode = 'p';
	cout << "Point drawing mode." << endl;
	buttonDown = false;
	StopDrawingPolygonOrBezier();
	
	pointButton->setDown( TRUE );
	lineButton->setDown( FALSE );
	polygonButton->setDown( FALSE );
	bezierButton->setDown( FALSE );
}

// Line slot
// when Line button is pressed - switch to Line drawing mode
void Lab3DerivedShell::LineSlot()
{
	mode = 'l';
	cout << "Line drawing mode." << endl;
	buttonDown = false;
	StopDrawingPolygonOrBezier();
	
	lineButton->setDown( TRUE );
	pointButton->setDown( FALSE );
	polygonButton->setDown( FALSE );
	bezierButton->setDown( FALSE );
}

// Polygon slot
// when Polygon button is pressed - switch to Polygon drawing mode
void Lab3DerivedShell::PolygonSlot()
{
	mode = 'g';
	cout << "Polygon drawing mode." << endl;
	buttonDown = false;
	StopDrawingPolygonOrBezier();
	
	polygonButton->setDown( TRUE );
	pointButton->setDown( FALSE );
	lineButton->setDown( FALSE );
	bezierButton->setDown( FALSE );
}

// Polygon slot
// when Polygon button is pressed - switch to Polygon drawing mode
void Lab3DerivedShell::BezierSlot()
{
	mode = 'b';
	cout << "Bezier curve drawing mode." << endl;
	buttonDown = false;
	StopDrawingPolygonOrBezier();
	
	bezierButton->setDown( TRUE );
	polygonButton->setDown( FALSE );
	pointButton->setDown( FALSE );
	lineButton->setDown( FALSE );
}


// Non-zero Winding Algorithm Slot
// called when nzw check-box changes state
void Lab3DerivedShell::NzwSlot()
{
	buttonDown = false;
	StopDrawingPolygonOrBezier();
	
	if ( nonZeroWinding ) {
		nonZeroWinding = FALSE;
		cout << "Non-zero Winding algorithm is OFF." << endl;
	}
	else {
		nonZeroWinding = TRUE;
		cout << "Non-zero Winding algorithm is ON." << endl;
	}
	nzwCheckBox->setChecked( nonZeroWinding );
}

// Set color slot (color for lines and points)
// called when Color button is pressed
void Lab3DerivedShell::ColorSlot()
{
	buttonDown = false;
	StopDrawingPolygonOrBezier();
	
	QColor col = QColorDialog::getColor( color, 0, 0 );	// opens a dialog to select a color
	if ( col.QColor::isValid() ) {
		color = col;	// if no color is selected (e.g. Cancel is pushed)
		colorIndication->setBackgroundColor( color );
	}
}

// Set fill color Slot
// called when Fill Color button is called
void Lab3DerivedShell::FillColorSlot()
{
	buttonDown = false;
	StopDrawingPolygonOrBezier();
	
	QColor col = QColorDialog::getColor( brushColor, 0, 0 );	// opens a dialog to select a color
	if ( col.QColor::isValid() ) {
		brushColor = col;	// if no color is selected (e.g. Cancel is pushed)
		fillColorIndication->setBackgroundColor( brushColor );
	}
}

// Line style select box
// called when a line style is selected from the line style listbox
void Lab3DerivedShell::LineStyleSlot()
{
	buttonDown = false;
	StopDrawingPolygonOrBezier();
	
	penStyle = (Qt::PenStyle) (lineStyleBox->currentItem() + 1);	// zero index is noPen - not used in this program
	cout << "Line style changed." << endl;
}

// Fill style select box
// called when a filling style is selected from the fill style listbox
void Lab3DerivedShell::FillStyleSlot()
{
	buttonDown = false;
	StopDrawingPolygonOrBezier();
	brushStyle = (Qt::BrushStyle) fillStyleBox->currentItem();
	cout << "Filling style changed." << endl;
}

void Lab3DerivedShell::PanXSlot()
{
	StopDrawingPolygonOrBezier();
	// cout << "translation: dx = " << panX->value() << endl;
	viewContext->SetPosition( QPoint(-panX->value(), 0) );
	Refresh();
}

void Lab3DerivedShell::PanYSlot()
{
	StopDrawingPolygonOrBezier();
	// cout << "translation: dy = " << panY->value() << endl;
	viewContext->SetPosition( QPoint(0, panY->value()) );
	Refresh();
}

void Lab3DerivedShell::RotationSlot()
{
	StopDrawingPolygonOrBezier();
	// cout << "rotation: angle = " << rotation->value() << endl;
	viewContext->SetRotation( rotation->value() );
	Refresh();
}

// Cancel drawing a polygon
void Lab3DerivedShell::StopDrawingPolygonOrBezier()
{
	bool erased = false;
	if (drawingPolygon) {
		drawingPolygon = false;
		polygonButton->setText( "Polygon" );
		if ( pol_ptr != NULL ) {	// release memory if necessary
			delete pol_ptr;
			pol_ptr = NULL;
			drawArea->erase();
			act_image->ReDraw( drawArea, viewContext );	// redraw the screen to erase all temporary pixels (could be done by storing previous color and redraw only affected pixels)
			cout << "Canceled drawing a polygon." << endl;
			erased = true;
		}
	}
	if (drawingBezier) {
		drawingBezier = false;
		bezierButton->setText( "Bezier" );
		if ( bez_ptr != NULL ) {	// release memory if necessary
			delete bez_ptr;
			bez_ptr = NULL;
			if ( !erased ) {
				drawArea->erase();
				act_image->ReDraw( drawArea, viewContext );	// redraw the screen to erase all temporary pixels (could be done by storing previous color and redraw only affected pixels)
			}
			cout << "Canceled drawing a bezier curve." << endl;
		}
	}
}

// Draws temporary pixels which represent vertices of lines and polygons that are being created
// pixels are drawn to the screen, but not saved in memory
void Lab3DerivedShell::DrawTempPixel( int x, int y )
{
	painter.begin(drawArea);
	painter.setPen( color );
	painter.drawPoint( x, y );
	painter.end();
}

// refresh screen
// used after change of view (translation, rotation, scaling)
void Lab3DerivedShell::Refresh()
{
	// program in normal mode
	if ( !drawingPolygon && !drawingBezier ) {
		drawArea->erase();
		act_image->ReDraw( drawArea, viewContext );
	}
	// program in drawing polygon or bezier started state
	else {	
		StopDrawingPolygonOrBezier();	// it will erase and redraw image
		
	}
}

// Method to post resize events to standard callback
void Lab3DerivedShell::resizeEvent(QResizeEvent* resizeEvent)
{
}

// zoom button was pressed, change the scaling factor in viewContext and redraw screen
void Lab3DerivedShell::Zoom()
{
	StopDrawingPolygonOrBezier();
	viewContext->SetScale( 2 );
	Refresh();
}

// unzoom button was pressed, change the scaling factor in viewContext and redraw screen
void Lab3DerivedShell::Unzoom()
{
	StopDrawingPolygonOrBezier();
	viewContext->SetScale( 0.5 );
	Refresh();
}

// reset button was pressed, change scaling factor and translation to make whole picture fit in screen
void Lab3DerivedShell::Reset()
{
	/* it will call ReDraw image method, while the reseting flag in viewContext is set, which makes the shapes pretend they are
	 * drawing, but in fact, they only compute their coordinates and do not draw anything to screen;
	 * after that optimal view transformation is computed and picture is really redrawn
	 */
	StopDrawingPolygonOrBezier();
	viewContext->SetReseting( true );
	act_image->ReDraw( drawArea, viewContext );	
	viewContext->ComputeReseting( areaWidth, areaHeight );
	viewContext->SetReseting( false );
	// move sliders to adequate values without calling their slots
	panX->blockSignals( true );
	panY->blockSignals( true );
	panX->setValue( -viewContext->GetXpos() );
	panY->setValue( viewContext->GetYpos() );
	panX->blockSignals( false );
	panY->blockSignals( false );
	Refresh();
}