/* Marek Handl
 * CS321 - Computer Graphics
 * Lab 5
 * October 2006
 */

#ifndef LAB3DERIVEDSHELL_H
#define LAB3DERIVEDSHELL_H

#include <shell.h>
#include <vector.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qcolor.h>
#include <qpointarray.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qhbox.h>
#include <qslider.h>
#include "image.h"
#include "variable_shape.h"
#include "polygon.h"
#include "bezier.h"
#include "ViewContext.h"

class Lab3DerivedShell : public Shell
{
Q_OBJECT

	protected:
	QPushButton *pointButton, *lineButton, *polygonButton, *bezierButton;	// pointers to buttons (used to push them down visualy)
	QCheckBox *nzwCheckBox;		// non-zero winding algorithm check box
	QComboBox *lineStyleBox, *fillStyleBox;
	QPainter painter;	// used to draw temporary pixels which represent vertices of lines and polygons that are being created
	QHBox *colorIndication, *fillColorIndication;
	QSlider *panX, *panY, *rotation;
	QPoint bezierLastPoint;	
	int areaWidth;
	int areaHeight;

	void StopDrawingPolygonOrBezier();		// if an event occurs, that cancels drawing a polygon
	void DrawTempPixel( int x, int y );	// draws temporary pixels which represent vertices of lines and polygons that are being created
	void Refresh();
	
	vector<image*> images;	// collection of all images
	image* act_image;	// image which is active, all the drawings is happening in this image
	char mode;		// in which drawing mode program is
	int start_x, start_y;	// start point for a line
	bool buttonDown;	// there was a MouseButtonDown event
	bool drawingPolygon;	// user started drawing polygon
	bool drawingBezier;
	bool nonZeroWinding;
	Qt::PenStyle penStyle;
	Qt::BrushStyle brushStyle;
	QColor color, brushColor;
	polygon *pol_ptr;	// pointer to a polygon which is being created
	bezier *bez_ptr;
	ViewContext *viewContext;

	
	protected slots:
	void PointSlot();
	void LineSlot();
	void PolygonSlot();
	void NzwSlot();
	void ColorSlot();
	void FillColorSlot();
	void LineStyleSlot();
	void FillStyleSlot();
	void PanXSlot();
	void PanYSlot();
	void RotationSlot();
	void BezierSlot();

	public:
	Lab3DerivedShell( QWidget* parent = NULL, const std::string& name="" ); // constructor
	~Lab3DerivedShell();	// destructor
	
	void Erase();
	void Zoom();
	void Unzoom();
	void Reset();
	bool CmdParse(const std::string& cmd);
	void Draw(QEvent* event);
	void ReadFile(const std::string& file);
	void WriteFile(const std::string& file) const;
	std::string HelpText() const;
	int GetWidth();
	int GetHeight();

	// Method to post resize events to standard callback
	void resizeEvent(QResizeEvent* resizeEvent);
	
};

#endif