/* Marek Handl
 * CS321 - Computer Graphics
 * Lab 5
 * October 2006
 */

#include "ViewContext.h"

// default constructor
ViewContext::ViewContext()
{
	scaleFactor = 1;
	theta = 0;
	xpos = 0;
	ypos = 0;
	T.SetSize(3,3);
	MakeIdentityMatrix( T );
	S.SetSize(3,3);
	MakeIdentityMatrix( S );
	transMatrix.SetSize(3,3);
	ComputeMatrices();
	invMatrix.SetSize(3,3);
	reseting = false;
}

// destructor
ViewContext::~ViewContext() {
	// nothing to be done here
	// there is no dynamic memory allocated
}

// makes matrix an identity matrix
void ViewContext::MakeIdentityMatrix( matrix<float> &m )
{
	for ( int i=0; i<m.RowNo(); i++ ) {
		for ( int j=0; j<m.ColNo(); j++ ) {
			m(i,j) = ( i==j ? 1 : 0 );
		}
	}
}

// computes transformation matrix and its inverse
void ViewContext::ComputeMatrices()
{
	transMatrix = T * S;
	invMatrix = !transMatrix;
}

// computes model coordinates when given window coordinates
QPoint ViewContext::WindowToModel( QPoint modelPoint ) const
{
	matrix<float> m(3,1);
	m(0,0) = modelPoint.x();
	m(1,0) = modelPoint.y();
	m(2,0) = 1;
	m = invMatrix * m;
	return QPoint( m(0,0), m(1,0) );
}

// computes window coordinates when given model coordinates
QPoint ViewContext::ModelToWindow( QPoint modelPoint )
{
	matrix<float> m(3,1);
	m(0,0) = modelPoint.x();
	m(1,0) = modelPoint.y();
	m(2,0) = 1;
	m = transMatrix * m;
	if ( reseting ) {	// if reset button was pressed, find min and max values of x and y on screen
		if ( m(0,0) < min_x )
			min_x = m(0,0);
		if ( m(0,0) > max_x )
			max_x = m(0,0);		
		if ( m(1,0) < min_y )
			min_y = m(1,0);
		if ( m(1,0) > max_y )
			max_y = m(1,0);
	}
	return QPoint( m(0,0), m(1,0) );
}

// write rotation to matrices
void ViewContext::SetRotation( double th )
{
	th = th/180 * 3.141592653589;
	theta = th;
	T(0,0) = T(1,1) = cos(theta);
	T(1,0) = sin(theta);
	T(0,1) = -T(1,0);
	ComputeMatrices();
}

// write translation to matrices
void ViewContext::SetPosition( QPoint p )
{
	if ( p.x() != 0 ) {	// translation in x coordinate
		xpos = p.x();
		T(0,2) = xpos;
	}
	if ( p.y() != 0 ) {	// translation in y coordinate
		ypos = p.y();
		T(1,2) = ypos;
	}
	ComputeMatrices();
}

// write scaling factor to matrices
void ViewContext::SetScale( double sf )
{
	scaleFactor *= sf;
	S(0,0) = S(1,1) = scaleFactor;	
	ComputeMatrices();
}

// set all matrices to default values - reset all translations, rotations and scaling
void ViewContext::Reset()
{
	scaleFactor = 1;
	theta = 0;
	xpos = 0;
	ypos = 0;
	MakeIdentityMatrix( T );
	MakeIdentityMatrix( S );
	ComputeMatrices();
}

// set or reset flag that makes the draw method find min and max values for x and y
void ViewContext::SetReseting( bool res ) {
	reseting = res;
	if ( res ) {
		min_y = min_x = 99999999;
		max_x = max_y = -99999999;
	}
}

// computes necessary translation and scaling for best fit of image to window
void ViewContext::ComputeReseting( int width, int height ) {
	width--;
	height--;
	double s_x = (double)width / (double)(max_x-min_x);
	double s_y = (double)height / (double)(max_y-min_y);
	double scale = min( s_x, s_y );
	xpos = (double)( width - scale*(max_x+min_x) ) / 2 + scale*xpos;
	T(0,2) = xpos;
	ypos = (double)( height - scale*(max_y+min_y) ) / 2 + scale*ypos;
	T(1,2) = ypos;
	SetScale( scale );	// this will also compute all the matrices
}

// getters for private members
double ViewContext::GetTheta() const
{
	return theta;
}
double ViewContext::GetXpos() const
{
	return xpos;
}
double ViewContext::GetYpos() const
{
	return ypos;
}
bool ViewContext::GetReseting() const
{
	return reseting;
}