/* Marek Handl
 * CS321 - Computer Graphics
 * Lab 5
 * October 2006
 */

#include "bezier.h"
#include <iostream>
#include <qpainter.h>
#include <qcolor.h>
#include <qbrush.h>
#include <cmath>	// pow
#include <cfloat>	// DBL_EPSILON

using namespace std;

// constructor
bezier::bezier( int numFacets, Qt::PenStyle ps, QColor col ) : VariableShape('b', ps, col) {
	// at first shape constructor is called
	// array of vertices is created in VariableShape constructor
	numberOfFacets = numFacets;
	counter++;			// static counter, each created shape increases it
	counter_value = counter;	// shape ID = state of counter at the time of creation of the shape
}	

// copy-constructor
bezier::bezier( const bezier& b ) : VariableShape('b', b.penStyle, b.color) {
	// at first VariableShape constructor is called
	// vertices are copied in VariableShape copy-constructor
	counter++;			// static counter, each created shape increases it
	counter_value = counter;	// shape ID = state of counter at the time of creation of the shape
}

// destructor
bezier::~bezier() {
	cout << "destroying bezier #" << counter_value << endl;
}

// draw itself on screen
void bezier::Draw( QWidget *drawArea, ViewContext *viewContext ) const {
	//cout << "drawing a bezier <" << x0 << ", " << y0 << ">, <" << x1 << ", " << y1 << ">" << " #" << counter_value << endl;
	QPoint startPoint( viewContext->ModelToWindow( vertices->point(0) ) );
	QPoint endPoint;
	QPainter painter( drawArea );
	// do this only when we want to draw to screen and not only determine min and max coordinates
	if ( !viewContext->GetReseting() ) {
		QPen pen;
		pen.setStyle( Qt::SolidLine );
		pen.setColor( color );
		painter.setPen( pen );
	}	
	// only one vertex -> it is a point, and we want to draw to screen (GetReseting returns 0)
	if ( vertices->size() == 1 && !viewContext->GetReseting() ) {
		painter.drawPoint( startPoint );
		return;
	}
	// only two vertices -> it is a line
	if ( vertices->size() == 2 ) {
		endPoint = QPoint( viewContext->ModelToWindow( vertices->point(1) ) );
		if ( !viewContext->GetReseting() ) painter.drawLine( startPoint, endPoint );	// true when we want to draw to screen
		return;
	}
	// more than 2 vertices
	// at first draw the polyline
	for ( int i=1; i<vertices->size(); i++ ) {
		endPoint = viewContext->ModelToWindow( vertices->point(i) );
		if ( !viewContext->GetReseting() ) painter.drawLine( startPoint, endPoint );	// true when we want to draw to screen
		startPoint = endPoint;
	}
	// do this only when we want to draw the curve to screen
	if ( !viewContext->GetReseting() ) {
		startPoint = viewContext->ModelToWindow( vertices->point(0) );
		double dmu = 1/(double)numberOfFacets;
		double mu = 0;
		// draw the curve
		// first facet starts at the first user added point
		for ( int i=1; i<numberOfFacets; i++ ) {
			mu += dmu;
			endPoint = viewContext->ModelToWindow( getPoint( mu ) );	// do the math to get another point
			painter.drawLine( startPoint, endPoint );	// draw a facet
			startPoint = endPoint;
		}
		// last facet of the curve, it ends at the last user added point
		endPoint = QPoint( viewContext->ModelToWindow( vertices->point( vertices->size()-1 ) ) );
		painter.drawLine( startPoint, endPoint );
	}
}

// write parameters to output file
void bezier::Write(std::ostream& ostr) const
{
	// output file is opened in main function
	shape::Write( ostr );		// write line style and color
	VariableShape::Write( ostr );	// write the vertices
	ostr << numberOfFacets << endl;
}

// read parameters from input file
void bezier::Read(std::istream& istr)
{
	// input file is opened in main function
	shape::Read( istr );		// read line style and color
	VariableShape::Read( istr );	// read vertices
	istr >> numberOfFacets;
}

// make a clone of itself
bezier* bezier::Clone() const
{
	// creates a new instance of the object with same parameters except shape ID (uses copy-constructor)
	return new bezier(*this);
}

// overloaded assignment operator
bezier& bezier::operator= (const bezier& b)
{
	if ( this == &b ) return *this;
	VariableShape::operator=( b );	// it will call shape operator= too
	return *this;
}



// Bezier interpolation wrapper
// Input arguments:
//	cp: control points
//	mu: Bezier parameter
// Returns:
//	P(mu), the requested point along the curve
QPoint bezier::getPoint( double mu ) const
{
	// This function will be called by your Draw function to calculate facet vertices along the curve.
	switch( vertices->size() )
	{
		case 3 : return getPoint3( mu );// You must complete this function, making use of the provided
		// utility functions (getPoint3, getPoint4, getPointN).
		case 4 : return getPoint4( mu );
		default: return getPointN( mu );
	}
}


// code from someone else
// Author: Dr. Eric Durant <durant@msoe.edu>
// Core calculation code based on work of Paul Bourke
// http://astronomy.swin.edu.au/~pbourke/curves/bezier/


// You should not need to modify anything below here (except the cp argument if you 
// use the second method listed above), but make sure you read and understand all
// the comments.

// Three point Bezier interpolation
// Input arguments:
//	cp: 3 control points
//	mu: Bezier parameter, ranges from 0 to 1, start to end of the curve
// Returns:
//	P(mu), the requested point along the curve
// Based on code by Paul Bourke
// http://astronomy.swin.edu.au/~pbourke/curves/bezier/
QPoint bezier::getPoint3( double mu ) const
{
	assert(vertices->size() == 3);
	// Common expressions for convenience: mu^2, 1-mu, (1-mu)^2
	const double mu2 = mu * mu, mum1 = 1 - mu, mum12 = mum1 * mum1;
	// Apply the quadratic Bezier equations
	return QPoint(	static_cast<int>(vertices->point(0).x() * mum12 + 2 * vertices->point(1).x() * mum1 * mu + vertices->point(2).x() * mu2),
			static_cast<int>(vertices->point(0).y() * mum12 + 2 * vertices->point(1).y() * mum1 * mu + vertices->point(2).y() * mu2)
		     );
}

// Four point Bezier interpolation
// Input arguments:
//	cp: 4 control points
//	mu: Bezier parameter, ranges from 0 to 1, start to end of the curve
// Returns:
//	P(mu), the requested point along the curve
// Based on code by Paul Bourke
// http://astronomy.swin.edu.au/~pbourke/curves/bezier/
QPoint bezier::getPoint4( double mu ) const
{
	assert(vertices->size() == 4);
	// Common expressions for convenience: 1-mu, (1-mu)^3, mu^3
	const double mum1 = 1 - mu, mum13 = mum1 * mum1 * mum1, mu3 = mu * mu * mu;
	// Apply the cubic Bezier equations
	return QPoint(	static_cast<int>(mum13*vertices->point(0).x() + 3*mu*mum1*mum1*vertices->point(1).x()
						+ 3*mu*mu*mum1*vertices->point(2).x() + mu3*vertices->point(3).x()),
			static_cast<int>(mum13*vertices->point(0).y() + 3*mu*mum1*mum1*vertices->point(1).y()
						+ 3*mu*mu*mum1*vertices->point(2).y() + mu3*vertices->point(3).y())
		     );
}

// Generalized Bezier interpolation
// Input arguments:
//	cp: control points
//	mu: Bezier parameter, ranges from 0 to 1-, start almost to end of the curve
//		IMPORTANT: Unstable as mu becomes very close to 1
// Returns:
//	P(mu), the requested point along the curve
// Based on code by Paul Bourke
// http://astronomy.swin.edu.au/~pbourke/curves/bezier/
QPoint bezier::getPointN( double mu ) const
{
	assert(mu>=0.0 && mu<=1.0); // Ensure mu is in range
	assert(1-mu >= DBL_EPSILON * 10.0); // Unstable as mu->1 -- Caller should just use final point

	// See Paul Bourke's web site for algorithm details...
	double blend; // the contribution (between 0 and 1) of a particular control point
	double muk = 1.0;
	double munk = pow(1-mu,static_cast<double>(vertices->size()-1));
	QPointArray::ConstIterator cit = vertices->begin();
	double bx = 0.0, by = 0.0;
	for (unsigned int idxk = 0; idxk < vertices->size(); idxk++, cit++)
	{
		unsigned int nn = vertices->size() - 1;
		unsigned int kn = idxk;
		unsigned int nkn = vertices->size() - 1 - idxk;
		blend = muk * munk;
		muk *= mu;
		munk /= (1-mu);
		while (nn >= 1) {
			blend *= nn;
			nn--;
			if (kn > 1) {
				blend /= static_cast<double>(kn);
				kn--;
			}
			if (nkn > 1) {
				blend /= static_cast<double>(nkn);
				nkn--;
			}
		}
		bx += cit->x() * blend;
		by += cit->y() * blend;
	}

	return QPoint(static_cast<int>(bx), static_cast<int>(by));
}
