/* Marek Handl
 * CS321 - Computer Graphics
 * Lab 5
 * October 2006
 */

#include <iostream>
#include <qwidget.h>
#include <qpointarray.h>
#include "image.h"
#include "shape.h"
#include "point.h"
#include "line.h"
#include "variable_shape.h"
#include "polygon.h"
#include "bezier.h"
/*#include "rectangle.h"
#include "circle.h"*/

using namespace std;

// constructor
image::image(string Name) 
{
	name = Name;
}

// copy constructor
image::image( const image& source)
{
	shape *sh_ptr;
	name = source.name;
	for ( unsigned i=0; i<source.shapes.size(); i++ ) {
		sh_ptr = source.shapes[i]->Clone();	// create new instance of each shape
		shapes.push_back( sh_ptr );		// put the new clone in vector
	}
	name = "copy_of_" + source.name;
}

// overloaded assignment operator
image& image::operator= (const image& source)
{
	Erase();	// delete all shapes contained in image
	shape *sh_ptr;
	for ( unsigned i=0; i<source.shapes.size(); i++ ) {
		sh_ptr = source.shapes[i]->Clone();	// create new instance of each shape
		shapes.push_back( sh_ptr );		// put the new clone in vector of target image
	}
	name = "duplicate_of_" + source.name;
	return *this; 
}
	
// destructor
image::~image()
{
	Erase();	// delete all shapes contained in image
}
	
// returns number of shapes in the image
int image::NumberOfShapes()
{
	return shapes.size();
}

// add a shape to the image
void image::Add(shape* sh_ptr)
{
	// the instance of shape is already created (done in main function), this only adds pointer to the shape to the vector of shapes
	shapes.push_back( sh_ptr );
}

// draw all the shapes in the image
void image::ReDraw( QWidget *drawArea, ViewContext *viewContext ) const
{
	// go through the vector of shapes and let each shape draw itself
	for ( unsigned i=0; i<shapes.size(); i++ ) {
		shapes[i]->Draw( drawArea, viewContext );
	}
}

// output image shapes to file
void image::Write(std::ostream& ostr) const
{
	// go through the vector of shapes and let each shape write its parameters to output file (name of output file is read in main function)
	for ( unsigned i=0; i<shapes.size(); i++ ) {
		shapes[i]->Write( ostr );
		ostr << endl;
	}
}	

// input shapes from file into a image
void image::Read(std::istream& istr)
{
	// shapes are created using their constructors with some default values
	// then is Read function of each shape called
	// at first common shape parameters (e.g. color) are read
	// then parameters significat for the concrete shape are read from file
	// pointers to new created shapes are added to vector of shapes
	char shape_type;
	string s;
	shape *sh_ptr;
	QPointArray *v = NULL;
	while ( getline( istr, s ) ) {	
		shape_type = s[0];
		switch (shape_type) {
			case 'p':	// Point
				sh_ptr = new point( 0, 0, Qt::SolidLine, QColor(255,255,255) );
				break;
			case 'l':	// Line
				sh_ptr = new line( 0, 0, 0, 0, Qt::SolidLine, QColor(255,255,255) );
				break;
			case 'g':	// Polygon
				sh_ptr = new polygon( TRUE, Qt::SolidLine, QColor(255,255,255), Qt::SolidPattern, QColor(255,255,255) );
				break;
			case 'b':	// Bezier Curve
				sh_ptr = new bezier( 100, Qt::SolidLine, QColor(255,255,255) );
				break;			
			default:
				if ( s == "" ) continue;
				cout << "Input file corrupted!" << endl;
				sh_ptr = NULL;
				break;
		}
		if ( sh_ptr == NULL ) {		// the file was corrupted, cancel reading from file
			break;
		}
		sh_ptr->Read( istr );	// call appropriate function for reading from file
		Add( sh_ptr );		// add new shape to the vector of shapes
	}
}		

// erase image
void image::Erase()
{
	// deletes all shapes from the image
	// the image object stays
	for ( unsigned i=0; i<shapes.size(); i++ ) {
		delete shapes[i];	// calls appropriate destructor for each shape
	}
	shapes.clear();		// reset the vector of shapes
}


// not implemented so far
// Methods for changing an image
void image::Zoom() {}		// Zoom the image
void image::Unzoom() {}		// Unzoom the image
void image::Reset() {}		// Reset the image
