/* Marek Handl
 * CS321 - Computer Graphics
 * Lab 5
 * October 2006
 */

//*************************************************
//
// image.h - Declares a container class for groups of
//	     graphical shapes
//
// Written by: H. Welch 9/10/98
//
// Modified by: H. Welch 9/14/98
//		Switched to storage of pointers
//
// Modified by: H. Welch 9/2/99
//		Changed Expose to ReDraw to avoid name
//		conflicts with X
//
// Modified by: H. Welch 9/14/99
//		Removed C style void parameters
//
// Modified by: H. Welch/4/2001
//		Added const to Redraw and Write to set a proper example
//
// Modified by: E. Durant 9/26/2003
//		Removed "using" for better portability
//
// Modified by: E. Durant 9/12/2004
//		Removed "Xform" for implementation flexibility
//*************************************************

#ifndef IMAGE_H
#define IMAGE_H

#include <vector>
#include "shape.h"

using namespace std;

// image object has a name and a vector of various shapes which the image contains
class image
{
	private:
	vector<shape*> shapes;			// pointers to all shapes in image
	
	public:
	string name;				// name of image
	int NumberOfShapes();

	image(string Name);			// Constructor
	image(const image& im);			// Copy constructor
	image& operator= (const image& source);	// Assignment =
	
	~image();				// Destructor
	
	// Methods for management
	void Add(shape* sh_ptr);		// Add shape to image

	// Methods for I/O
	void ReDraw( QWidget *drawArea, ViewContext *viewContext ) const;	// Draw all the shapes in the image
	void Write(std::ostream& ostr) const;	// Output image shapes to file
	void Read(std::istream& istr);		// Input image shapes from file

	// Methods for changing an image
	void Zoom();				// Zoom the image
	void Unzoom();				// Unzoom the image
	void Reset();				// Reset the image
	void Erase();				// Erase the image
};

#endif
