/* Marek Handl
 * CS321 - Computer Graphics
 * Lab 5
 * October 2006
 */

#include "line.h"
#include <iostream>
#include <qpainter.h>
#include <qcolor.h>

using namespace std;

// constructor
line::line( int X0, int Y0, int X1, int Y1, Qt::PenStyle ps, QColor col ) : shape('l', ps, col) {
	// at first shape constructor is called
	x0 = X0;
	y0 = Y0;
	x1 = X1;
	y1 = Y1;
	counter++;			// static counter, each created shape increases it
	counter_value = counter;	// shape ID = state of counter at the time of creation of the shape
}	

// copy-constructor
line::line( const line& l ) : shape('l', l.penStyle, l.color) {
	// at first shape constructor is called
	x0 = l.x0;
	y0 = l.y0;
	x1 = l.x1;
	y1 = l.y1;
	counter++;			// static counter, each created shape increases it
	counter_value = counter;	// shape ID = state of counter at the time of creation of the shape
}

// destructor
line::~line() {
	cout << "destroying line <" << x0 << ", " << y0 << ">, <" << x1 << ", " << y1 << ">"  << " #" << counter_value << endl;
}

// draw itself on screen
void line::Draw( QWidget *drawArea, ViewContext *viewContext ) const {
	//cout << "drawing a line <" << x0 << ", " << y0 << ">, <" << x1 << ", " << y1 << ">" << " #" << counter_value << endl;
	QPoint point0 = viewContext->ModelToWindow( QPoint(x0,y0) );
	QPoint point1 = viewContext->ModelToWindow( QPoint(x1,y1) );
	// if GetReseting() returns 0, no drawing to screen will be performed, this call has been made to find out coordinates of this shape only 
	if ( !viewContext->GetReseting() ) {
		QPainter painter;
		QPen pen;
		pen.setStyle( penStyle );
		pen.setColor( color );
	
		// if slope of line is 0 < m < 1 in viewing coordinates, use Bresenham algorithm
		if ( point1.y() > point0.y() && point1.x() > point0.x() && (point1.y()-point0.y()) < (point1.x()-point0.x()) ) {
			int dy = point1.y() - point0.y();
			int dx = point1.x() - point0.x();
			int act_x = point0.x();
			int act_y = point0.y();
			int p = 2*dy - dx;
			while ( act_x <= point1.x() ) {	// do it for each x coordinate between x0 and x1
				painter.begin( drawArea );
				painter.setPen( pen );
				painter.drawPoint( act_x, act_y );
				p += 2*dy;
				act_x++;
				if ( p > 0 ) {	// next pixel will be to the right and down, otherwise just to the right
					p -= 2*dx;
					act_y++;
				}
				painter.end();
			}
		}
		else {
			painter.begin( drawArea );
			painter.setPen( pen );
			painter.drawLine( point0, point1 );
			painter.end();
		}	
	}
}

// write parameters to output file
void line::Write(std::ostream& ostr) const
{
	// output file is opened in main function
	shape::Write( ostr );
	ostr << x0 << " " << y0 << " " << x1 << " " << y1;
}

// read parameters from input file
void line::Read(std::istream& istr)
{
	// input file is opened in main function
	shape::Read( istr );
	istr >> x0 >> y0 >> x1 >> y1;
}

// make a clone of itself
line* line::Clone() const
{
	// creates a new instance of the object with same parameters except shape ID (uses copy-constructor)
	return new line(*this);
}

// overloaded assignment operator
line& line::operator= (const line& l)
{
	if ( this == &l ) return *this;
	shape::operator=( l );
	x0 = l.x0;
	y0 = l.y0;
	x1 = l.x1;
	y1 = l.y1;
	return *this;
}
