/* Marek Handl
 * CS321 - Computer Graphics
 * Lab 5
 * October 2006
 */

#include "point.h"
#include "shape.h"
#include <iostream>
#include <qpainter.h>
#include <qwidget.h>

using namespace std;

// constructor
point::point( int X, int Y, Qt::PenStyle ps, QColor col ) : shape('p', ps, col) {
	// at first shape constructor is called
	x = X;
	y = Y;
	counter++;			// static counter, each created shape increases it
	counter_value = counter;	// shape ID = state of counter at the time of creation of the shape
}

// copy-constructor
point::point( const point& p ) : shape('p', p.penStyle, p.color) {
	// at first shape constructor is called
	x = p.x;
	y = p.y;
	counter++;			// static counter, each created shape increases it
	counter_value = counter;	// shape ID = state of counter at the time of creation of the shape
}

// destructor
point::~point() {
	cout << "destroying point <" << x << ", " << y << "> #" << counter_value << endl;	
}

// draw itself on screen
void point::Draw( QWidget *drawArea, ViewContext *viewContext ) const {
	//cout << "drawing a point <" << x << ", " << y << "> #" << counter_value << endl;	
	QPoint viewCoordinates;
	viewCoordinates = viewContext->ModelToWindow( QPoint(x,y) );
	// if GetReseting() returns 1, no drawing to screen will be performed, this call has been made to find out coordinates of this shape only
	if ( !viewContext->GetReseting() ) {
		QPainter p( drawArea );
		QPen pen;
		pen.setStyle( penStyle );
		pen.setColor( color );
		p.setPen( pen );
		p.drawPoint( viewCoordinates.x(), viewCoordinates.y() );
	}
}

// write parameters to output file
void point::Write(std::ostream& ostr) const
{
	// output file is opened in main function
	shape::Write( ostr );
	ostr << x << " " << y;	
}

// read parameters from input file
void point::Read(std::istream& istr)
{
	// input file is opened in main function
	shape::Read( istr );
	istr >> x >> y;
}

// make a clone of itself
point* point::Clone() const
{
	// creates a new instance of the object with same parameters except shape ID (uses copy-constructor)
	return new point(*this);	
}

// overloaded assignment operator
point& point::operator= (const point& p)
{
	if ( this == &p ) return *this;
	shape::operator=( p );
	x = p.x;
	y = p.y;
	return *this;	
}

