/* Marek Handl
 * CS321 - Computer Graphics
 * Lab 5
 * October 2006
 */

#include "polygon.h"
#include <iostream>
#include <qpainter.h>
#include <qcolor.h>
#include <qbrush.h>

using namespace std;

// constructor
polygon::polygon( bool nzw, Qt::PenStyle ps, QColor col, Qt::BrushStyle bs, QColor bcol ) : VariableShape('g', ps, col) {
	// at first shape constructor is called
	// array of vertices is created in VariableShape constructor
	nonZeroWinding = nzw;
	brushColor = bcol;
	brushStyle = bs;
	counter++;			// static counter, each created shape increases it
	counter_value = counter;	// shape ID = state of counter at the time of creation of the shape
}	

// copy-constructor
polygon::polygon( const polygon& g ) : VariableShape('g', g.penStyle, g.color) {
	// at first VariableShape constructor is called
	// vertices are copied in VariableShape copy-constructor
	nonZeroWinding = g.nonZeroWinding;
	brushColor = g.brushColor;
	brushStyle = g.brushStyle;
	counter++;			// static counter, each created shape increases it
	counter_value = counter;	// shape ID = state of counter at the time of creation of the shape
}

// destructor
polygon::~polygon() {
	cout << "destroying polygon #" << counter_value << endl;
}

// draw itself on screen
void polygon::Draw( QWidget *drawArea, ViewContext *viewContext ) const {
	//cout << "drawing a polygon <" << x0 << ", " << y0 << ">, <" << x1 << ", " << y1 << ">" << " #" << counter_value << endl;
	QPointArray actualVertices;
	for (int i=0; i<vertices->size(); i++ ) {
		QPoint p = viewContext->ModelToWindow( vertices->at(i) );
		actualVertices.putPoints( i, 1, p.x(), p.y() );
	}
	// if GetReseting() returns 0, no drawing to screen will be performed, this call has been made to find out coordinates of this shape only 
	if ( !viewContext->GetReseting() ) {
		QPainter painter( drawArea );
		QPen pen;
		pen.setStyle( penStyle );
		pen.setColor( color );
		painter.setPen( pen );
		QBrush brush;
		brush.setColor( brushColor );
		brush.setStyle( brushStyle );
		painter.setBrush( brush );
		painter.drawPolygon( actualVertices, nonZeroWinding, 0, -1 );
	}
}

// write parameters to output file
void polygon::Write(std::ostream& ostr) const
{
	// output file is opened in main function
	shape::Write( ostr );		// write line style and color
	VariableShape::Write( ostr );	// write the vertices
	ostr << (nonZeroWinding ? "1" : "0");
	ostr << " " << (int) brushStyle << " " << brushColor.red() << " " << brushColor.green() << " " << brushColor.blue() << endl;
}

// read parameters from input file
void polygon::Read(std::istream& istr)
{
	// input file is opened in main function
	shape::Read( istr );		// read line style and color
	VariableShape::Read( istr );	// read vertices
	int bs, red, green, blue;
	istr >> nonZeroWinding >> bs >> red >> green >> blue;
	brushStyle = (Qt::BrushStyle) bs;
	brushColor.setRgb( red, green, blue );
}

// make a clone of itself
polygon* polygon::Clone() const
{
	// creates a new instance of the object with same parameters except shape ID (uses copy-constructor)
	return new polygon(*this);
}

// overloaded assignment operator
polygon& polygon::operator= (const polygon& g)
{
	if ( this == &g ) return *this;
	VariableShape::operator=(g);	// it will call shape operator= too
	// Polygon parameters
	nonZeroWinding = g.nonZeroWinding;
	brushColor = g.brushColor;
	brushStyle = g.brushStyle;
	return *this;
}
