/* Marek Handl
 * CS321 - Computer Graphics
 * Lab 5
 * October 2006
 */

//*************************************************
//
// $Id: shape.cpp,v 1.5 2004/09/12 22:27:38 durant Exp $
// Defines a base class for graphical shapes
// Virtually no functionality is included.
//
// Written by: H. Welch 9/10/98
//
// Modified by: H. Welch 9/14/98
//		Removed ordering operators and added Clone
//
// Modified by: H. Welch 9/14/99
//		Fixed base pointer destructor problem
//		Removed old-style C void parameter
//		Streamlined copy c-tor and Clone
//
// Modified by: H. Welch 5/10/2001
//		Updated namespacing to reflect modern practice
//		as allowed by the new compiler
//
// Modified by: E. Durant 9/17/2003
//		removed Xform method
//		qualified namespace usage
//
// Modified by: E. Durant 9/26/2003
//		Removed "using" for better portability
//
// Modified by: E. Durant 9/12/2004
//		Removed and documented functions that are now abstract
//		Changed type to char for readability
//*************************************************

#include <iostream>
#include <cassert>
#include <qpainter.h>
#include <qcolor.h>
#include "shape.h"

//*************************************************
//
// shape::shape - Constructors for base class shape
//
// Parameters: NONE or shape to copy
//
// Returns: NOTHING (the shape for assignment=)
//
// Written by: H. Welch 9/10/98
//
// Calls: NONE
//
//*************************************************
int shape::counter=0;

shape::shape(char theType, Qt::PenStyle ps, QColor col) {
	type = theType;
	penStyle = ps;
	color = col;
}

shape::shape(const shape& sh)
{
	type = sh.type;
	penStyle = sh.penStyle;
	color = sh.color;
}

shape& shape::operator= (const shape& r)
{
	type = r.type;
	penStyle = r.penStyle;
	color = r.color;
	return(*this);
}

//*************************************************
//
// shape::~shape - Destructor for base class shape
//		   (DOES NOTHING)
//
// Parameters: NONE
//
// Returns: NOTHING
//
// Written by: H. Welch 9/10/98
//
// Calls: NONE
//
//*************************************************

shape::~shape()
{ // Nothing to delete or free
}

// shape::Draw is not implemented since it is pure virtual (abstract).

//*************************************************
//
// shape::Write - Write shape to output file
//
// Parameters: ostr - the output stream to use
//
// Returns: NOTHING
//
// Written by: H. Welch 9/10/98
//
// Calls: iostream
//
//*************************************************
void shape::Write(std::ostream& ostr) const
{
	ostr << type << std::endl;
	ostr << (int) penStyle << " " << color.red() << " " << color.green() << " " << color.blue() << std::endl;
}

//*************************************************
//
// shape::Read - Read shape from input file
//		 Base object has no data so does nothing
//
// Parameters: istr - the input stream to use
//
// Returns: NOTHING
//
// Written by: H. Welch 9/10/98
//
// Calls: NONE
//
//*************************************************
void shape::Read(std::istream& istr)
{
	int red, green, blue, pens;
 	istr >> pens >> red >> green >> blue;
	penStyle = (Qt::PenStyle) pens;
	color.setRgb( red, green, blue );
}

// shape::Clone is not implemented since it is pure virtual (abstract).
// In derived classes, you should make use of your copy constructor...
// shape* theClone = new myDerivedType(*this);
// Check for null pointer and then return theClone.
