/* Marek Handl
 * CS321 - Computer Graphics
 * Lab 5
 * October 2006
 */

//*************************************************
//
// $Id: shape.h,v 1.5 2004/09/12 22:27:20 durant Exp $
// Declares a base class for graphical shapes
//
// Written by: H. Welch 9/10/98
//
// Modified by: H. Welch 9/14/98
//		Removed ordering operators and added Clone
//
// Modified by: H. Welch 9/14/99
//		Fixed base pointer destructor problem
//		Removed old-style C void parameters
//
// Modified by: H. Welch 5/10/2001
//		Updated namespacing to reflect modern practice
//		as allowed by the new compiler
//
// Modified by: E. Durant 9/17/2003
//		removed Xform method
//		qualified namespace usage
//
// Modified by: E. Durant 9/26/2003
//		Removed "using" for better portability 
//
// Modified by: E. Durant 9/12/2004
//		Made some virtual functions pure virtual (abstract) to support
//		better sharing of responsibility between base and derived classes.
//*************************************************

#ifndef SHAPE_H
#define SHAPE_H
#include <iostream>
#include <qpainter.h>
#include <qcolor.h>
#include <qwidget.h>
#include "ViewContext.h"

// abstract class, creates interface for e.g line. class
class shape
{
protected:
	char type;				// To support Read and Write
	static int counter;
	Qt::PenStyle penStyle;
	QColor color;
	
public:
	shape(char thetype, Qt::PenStyle ps, QColor col);			// Base ctor used by derived classes
						// There is no null ctor.
	shape(const shape& sh);			// copy-constructor
	shape& operator= (const shape& r);
	
	virtual ~shape();			// destructor
	
	// Virtual methods
	virtual void Draw( QWidget *drawArea, ViewContext *viewContext ) const = 0;		// Draw the shape
	virtual void Write(std::ostream& ostr) const;// Output shape to file
	virtual void Read(std::istream& istr);	// Input shape from file
	virtual shape* Clone() const = 0;	// Return a new copy
};

#endif
